export declare class AsyncScope<K, V> {
    readonly asyncId: number;
    protected _children?: Map<number, AsyncScope<K, V>>;
    protected _map?: Map<K, V>;
    protected _parent?: AsyncScope<K, V>;
    protected _destroyCb?: (asyncId: number) => void;
    protected _nameSet?: Set<string>;
    constructor(asyncId: number, parent?: AsyncScope<K, V>);
    hasName(name: string): boolean | undefined;
    alias(name: string): this;
    get(key: K): V | undefined;
    has(key: K, recursion?: boolean): boolean;
    set(key: K, value: V): this;
    delete(key: K): boolean;
    clear(): void;
    /**
     * get parent scope
     * if name provided , return the named closest scope
     * this method will throw an error if there is no parent
     *
     * @param {string} [name]
     * @returns {AsyncScope<K, V>}
     * @memberof AsyncScope
     */
    parent(name?: string): AsyncScope<K, V>;
    /**
     * alias of parent
     *
     * @param {string} name
     * @returns
     * @memberof AsyncStorage
     */
    closest(name: string): AsyncScope<K, V>;
    /**
     * check if the scope has children?
     *
     * @returns
     * @memberof AsyncScope
     */
    hasChildren(): boolean | undefined;
    /**
     * destroy the scope,the callback will be called when real destroyed
     *
     * @param {(asyncId: number) => void} cb
     * @memberof AsyncScope
     */
    destroy(cb: (cbId: number) => void): void;
    /**
     * real destroy the scope and call the callback
     *
     * @protected
     * @memberof AsyncScope
     */
    protected realDestroy(): void;
}
export declare class AsyncHookMap<K = any, V = any> {
    protected maps: Map<number, AsyncScope<K, V>>;
    protected hook: {
        enable(): void;
        disable(): void;
    };
    protected asyncHooks: any;
    /**
     * AsyncStorage
     * default backend by node 8+ async-hooks
     * @param {string} [asyncHooks=require('async_hooks')]
     * @memberof AsyncHookMap
     */
    constructor(asyncHooks?: any);
    /**
     * disable the AsyncHook,and desctroy all scope
     *
     */
    desctroy(): void;
    /**
     * alias of asyncHooks.executionAsyncId()
     *
     * @returns {number}
     */
    executionAsyncId(): number;
    /**
     * add alias of scope, a scope can own multi names
     *
     * @param {string} name
     * @returns {this}
     */
    alias(name: string): this;
    /**
     * check the alias name
     *
     * @param {string} name
     * @returns {this}
     */
    hasName(name: string): boolean | undefined;
    /**
     * get parent scope
     * if name provided , return the named closest scope
     * this method will throw an error if there is no parent
     *
     * @param {string} [name]
     * @returns {TinyMap<K, V>}
     * @memberof AsyncStorageInterface
     */
    parent(name?: string): AsyncScope<K, V>;
    /**
     * alias of parent
     *
     * @param {string} name
     * @returns
     * @memberof AsyncStorage
     */
    closest(name: string): AsyncScope<K, V>;
    /**
     * get from AsyncStorage
     *
     * @param {K} key
     * @returns {(V | undefined)}
     * @memberof AsyncStorage
     */
    get(key: K): V | undefined;
    /**
     * check key
     * @param key
     * @param recursion check forefathers?
     */
    has(key: K, recursion?: boolean): boolean;
    /**
     * set value to current async scope
     * effect current scope and children
     *
     * @param {K} key
     * @param {V} value
     * @returns {this}
     * @memberof AsyncStorage
     */
    set(key: K, value: V): this;
    /**
     * delete the value of current scope
     *
     * @param {K} key
     * @returns {boolean}
     * @memberof AsyncStorage
     */
    delete(key: K): boolean;
    /**
     * clear the current scope
     *
     * @memberof AsyncStorage
     */
    clear(): void;
    protected current(): AsyncScope<K, V>;
}
export default AsyncHookMap;
