"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
require("mocha");
const assert_1 = __importDefault(require("assert"));
const main_1 = require("../main");
// function print (...args: any[]) {
//     require('fs').writeFileSync(1, args.map(s => JSON.stringify(s)).join(' ') + '\n')
// }
describe('Test AsyncHookMap', () => {
    it('simple', () => {
        const scope = new main_1.AsyncHookMap();
        scope.set('aa', 11);
        const aa = scope.get('aa');
        assert_1.default(aa === 11, 'should be equal');
        process.nextTick(() => {
            assert_1.default(scope.get('aa') === 11, 'should be equal');
        });
    });
    it('test async scopes', async () => {
        return Promise.resolve().then(() => {
            const scope = new main_1.AsyncHookMap();
            scope.set('aa', 'first');
            scope.alias('ccc');
            assert_1.default.equal(scope.get('aa'), 'first');
            return Promise.all([
                Promise.resolve().then(() => {
                    assert_1.default.equal(scope.get('aa'), 'first');
                    scope.set('aa', 'second');
                    assert_1.default.equal(scope.get('aa'), 'second');
                }),
                Promise.resolve().then(() => {
                    assert_1.default.equal(scope.get('aa'), 'first', 'should not overwrite by another scope');
                }),
            ]);
        });
    });
    it('test named scopes and advance usages', async () => {
        return Promise.resolve().then(() => {
            const scope = new main_1.AsyncHookMap();
            scope.set('aa', 'first');
            scope.alias('ccc');
            assert_1.default.equal(scope.get('aa'), 'first');
            return Promise.resolve().then(() => {
                assert_1.default(scope.has('aa'), 'should has the key');
                assert_1.default(!scope.has('not'), 'should not has the key');
                assert_1.default(!scope.has('aa', false), 'should not has the key in this scope');
                assert_1.default.equal(scope.get('aa'), 'first');
                scope.set('aa', 'second');
                assert_1.default.equal(scope.get('aa'), 'second');
            }).then(() => {
                assert_1.default.equal(scope.get('aa'), 'second');
                assert_1.default.equal(scope.parent('ccc').get('aa'), 'first');
                // 'root' as alias of 'ccc'
                assert_1.default.equal(scope.parent('root').get('aa'), 'first');
                scope.parent().delete('aa');
                // parent scope 'aa' has been delete, 'aa' will be first
                assert_1.default.equal(scope.get('aa'), 'first');
                scope.parent('ccc').set('bb', 'bb');
                assert_1.default.equal(scope.get('bb'), 'bb');
                scope.delete('bb');
                // can not be deleted ,because bb is set to "ccc" scope
                assert_1.default.equal(scope.get('bb'), 'bb');
            });
        });
    });
});
//# sourceMappingURL=test_all.js.map